VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
END
Attribute VB_Name = "CRibbon"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = False
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
Option Explicit

' Class CRibbon
' Diese Klasse stellt Funktionen fr die Registerkarte "orgAnice" im Ribbon zur Verfgung.
' Sollte keine Ribbon vorhanden sein, so wird orgAncie-Men im Menpunkt "AddIns" genutzt

' orgAnice 2012
' (c) 2012-2023 orgAnice Software GmbH
' Autor: Steffen Mayer, Matthias Kozlowski, Xuan Guo
'
' Letzte nderung: 2023-07-07

' ffentliche Methoden:
' ShowRibbon
' ShowAddinMenu
' AddButton


Private Const csClassName As String = "CRibbon"

Private WithEvents mobjWordApplication As Word.Application
Attribute mobjWordApplication.VB_VarHelpID = -1
Private mbRibbonIsVisible As Boolean

Private Sub Class_Initialize()

    Set mobjWordApplication = Application

End Sub

Public Property Get RibbonVisible() As Boolean
    RibbonVisible = mbRibbonIsVisible
End Property

Public Function ShowRibbon() As Boolean

    Dim bSuccess As Boolean
    
    On Error GoTo ErrHandler
    
    bSuccess = SetRibbonVisibility(True)
    If bSuccess Then
        If Not (FindOrgAniceCommandBar Is Nothing) Then
            FindOrgAniceCommandBar.Visible = False
        End If
    Else
        ' Wenn das Ribbon nicht geladen werden kann,
        ' dann das herkmmliche AddIn-Men anzeigen
        ShowAddinMenu
        LocalizeOrgAniceCommandBar
    End If
    
    mbRibbonIsVisible = bSuccess
    ShowRibbon = bSuccess
    
    Exit Function
    
ErrHandler:
    Err.Raise Err.Number, csClassName & ".ShowRibbon"
End Function

Public Sub ShowAddinMenu()

    On Error GoTo ErrHandler
    
    If Not (FindOrgAniceCommandBar Is Nothing) Then
        FindOrgAniceCommandBar.Visible = True
    End If
    
    SetRibbonVisibility False
    mbRibbonIsVisible = False
    
    Exit Sub
    
ErrHandler:
    Err.Raise Err.Number, csClassName & ".ShowAddinMenu"
End Sub

Private Function SetRibbonVisibility(pbVisible As Boolean) As Boolean

    Dim objAddin As COMAddIn

    On Error GoTo ErrHandler

    Set objAddin = GetOrganiceComAddIn()
    
    If Not objAddin Is Nothing Then
        If pbVisible Then
            If Not objAddin.Connect Then
                objAddin.Connect = True
                objAddin.Object.SetRibbonVisibility True
            End If
        Else
            If objAddin.Connect Then
                objAddin.Connect = False
                objAddin.Object.SetRibbonVisibility False
            End If
        End If
    
        SetRibbonVisibility = True
    End If
    
    Exit Function
ErrHandler:
    SetRibbonVisibility = False
End Function

Private Function GetOrganiceComAddIn() As COMAddIn
    
    Dim i As Long
    For i = 1 To Application.COMAddIns.Count
        If gfDebug Then Debug.Print Application.COMAddIns(i).ProgID
        If Left(Application.COMAddIns(i).ProgID, 29) = "orgAnice.Components.WordAddIn" Then
            Set GetOrganiceComAddIn = Application.COMAddIns(i)
            Exit For
        End If
    Next i

End Function

Public Function AddButton(psLabel As String, psToolTip As String, psMacro As String, piGroup As Integer)

    Dim objAddin As COMAddIn
    
    Set objAddin = GetOrganiceComAddIn
    If mbRibbonIsVisible And Not objAddin Is Nothing Then
        objAddin.Object.AddButton psLabel, psToolTip, psMacro, piGroup
    Else
        With FindOrgAniceCommandBar.Controls.Add(Before:=GetMenuEntryPosition(piGroup))
            .Caption = psLabel
            .TooltipText = psToolTip
            .OnAction = psMacro
        End With
    End If

End Function

Private Function GetMenuEntryPosition(piGroup As Integer) As Integer

    Dim i As Long
    Dim objCommandBar As CommandBarPopup
    Dim sMacro As String

    Set objCommandBar = FindOrgAniceCommandBar

    Select Case piGroup
        Case 0
            sMacro = "Macros.EditBookmarks"
        Case 1:
            sMacro = "Macros.InfoBox"
    End Select

    For i = 1 To objCommandBar.Controls.Count
        If objCommandBar.Controls(i).OnAction = sMacro Then
            GetMenuEntryPosition = i
            Exit For
        End If
    Next i

End Function

Private Sub mobjWordApplication_Quit()
    
    If mbRibbonIsVisible Then
        SetRibbonVisibility False
    End If
    
End Sub

Private Sub LocalizeOrgAniceCommandBar()
 
    Dim orgMenu As office.CommandBarPopup
    Dim sTemplates() As String
    Dim i As Long
   
    On Error GoTo ErrHandler
   
    Set orgMenu = FindOrgAniceCommandBar
    
    If orgMenu Is Nothing Then
        Set orgMenu = CreateOrgAniceCommandBar
    End If
   
    i = LocalizeFirstControlWithoutTag(orgMenu.Controls, i + 1, TranslateCode(TRANSLATION_TYPE_ORGOFFICE, "orgAniceDb", "MenuSerialLetter"))
    i = LocalizeFirstControlWithoutTag(orgMenu.Controls, i + 1, TranslateCode(TRANSLATION_TYPE_ORGOFFICE, "orgAniceDb", "MenuSend"))
    i = LocalizeFirstControlWithoutTag(orgMenu.Controls, i + 1, TranslateCode(TRANSLATION_TYPE_ORGOFFICE, "orgAniceDb", "MenuAttachments"))
    i = LocalizeFirstControlWithoutTag(orgMenu.Controls, i + 1, TranslateCode(TRANSLATION_TYPE_ORGOFFICE, "orgAniceDb", "MenuDocumentBookmarks"))
    i = LocalizeFirstControlWithoutTag(orgMenu.Controls, i + 1, TranslateCode(TRANSLATION_TYPE_ORGOFFICE, "orgAniceDb", "MenuDocumentVariables"))
    i = LocalizeFirstControlWithoutTag(orgMenu.Controls, i + 1, TranslateCode(TRANSLATION_TYPE_ORGOFFICE, "orgAniceDb", "MenuInfo"))
 
    sTemplates = GetWordMacroTemplates
 
    For i = LBound(sTemplates) To UBound(sTemplates)
        SaveTemplate sTemplates(i)
    Next i
    
    Exit Sub
ErrHandler:
    ' weiter machen
End Sub
 
Private Function LocalizeFirstControlWithoutTag(pcolControls As CommandBarControls, plIndex As Long, psCaption As String)
 
    Dim i As Long
    i = plIndex
    Do While pcolControls(i).Tag <> ""
        i = i + 1
    Loop
    pcolControls(i).Caption = psCaption
    LocalizeFirstControlWithoutTag = i
 
End Function

Private Function FindOrgAniceCommandBar() As office.CommandBarPopup

    Dim cmdBar As CommandBar
    Dim i As Integer
    
    ' orgAnice Men suchen
    Set cmdBar = CommandBars("Menu Bar")
    For i = 1 To cmdBar.Controls.Count
        If Not cmdBar.Controls(i).BuiltIn Then
            If cmdBar.Controls(i).Caption = "&orgAnice" Then
                If cmdBar.Controls(i).Type = msoControlPopup Then
                    Set FindOrgAniceCommandBar = cmdBar.Controls(i)
                End If
            End If
        End If
    Next

End Function

Private Function CreateOrgAniceCommandBar() As office.CommandBarPopup

    Dim cmdBar As office.CommandBar
    Dim orgMenu As office.CommandBarControl
    
    Set cmdBar = CommandBars("Menu Bar")
    Set orgMenu = cmdBar.Controls.Add(Type:=msoControlPopup)
    orgMenu.Caption = "&orgAnice"
    
    With orgMenu.Controls.Add
        .OnAction = "PrintSerialLetter"
    End With

    With orgMenu.Controls.Add
        .OnAction = "PrintOneSerialLetter"
    End With
    
    With orgMenu.Controls.Add
        .OnAction = "SetAttachments"
    End With
    
    With orgMenu.Controls.Add
        .OnAction = "EditBookmarks"
    End With
    
    With orgMenu.Controls.Add
        .OnAction = "EditDocumentVariables"
    End With
    
    With orgMenu.Controls.Add
        .OnAction = "Infobox"
    End With

    Set CreateOrgAniceCommandBar = orgMenu

End Function

Private Function GetWordMacroTemplates() As String()

    Dim i As Long
    Dim sPaths() As String

    sPaths = Split(gOrgDb.Properties("WordMacroTemplates").Value, ";")
    
    For i = LBound(sPaths) To UBound(sPaths)
        sPaths(i) = GetAbsolutePath(sPaths(i))
    Next i

    GetWordMacroTemplates = sPaths

End Function

Private Function GetAbsolutePath(psPath As String) As String

    If Left(psPath, 1) = "." Then
        GetAbsolutePath = Left(gOrgDb.Filename, InStrRev(gOrgDb.Filename, "\") - 1) & Right(psPath, Len(psPath) - 1)
    Else
        GetAbsolutePath = psPath
    End If

End Function

Private Sub SaveTemplate(psName As String)
    
    Dim i As Long

    For i = 1 To Application.Templates.Count
        If Application.Templates(i).FullName = psName Then
            Application.Templates(i).Saved = True
            Exit Sub
        End If
    Next i

End Sub
